% Simulation Code for short-term Potentiation at hippocampal CA3-CA1 
% tripartite synapse. The present code makes use of MATLAB cell-mode which 
% enables the user to switch directly to the desired cell for making any 
% changes (e.g. switching astrocyte ON or OFF). This code is open source. 
% The reader is free to use, modify and distribute the code as long as 
% authorship is properly acknowledged: 
% Tewari, S. and Majumdar, K. (2012). "A Mathematical Model of Tripartite 
% Synapse: Astrocyte Induced Synaptic Plasticity," Journal of Biological
% Physics.
% For simulating 1 second of model time it takes ~6 seconds of real-time
% This code makes use of function 'marko.m' therefore make sure that it is 
% in the working directory of MATLAB or in its path.
%% Clear Memory & Display Start Time
clear all% Clears memory
Start_Time=clock;% Record start time
fprintf('%s %d %d %d','The Start Time is:',fix(Start_Time(4:6)),' (HH:MM:SS) and')% Display Start Time
tic;% To calculate total time elapsed it will be followed by 'toc' in the 
% last cell.

%% Time-duration of Simulation
Tmax=1.0e+3;% Time-duration of simulation; unit: ms
deltaT=0.05;% Fixed Time-step; unit: ms
tn=Tmax/deltaT;% Total number of time-points
t=0:deltaT:Tmax;% Time-vector

%% Bouton parameters
rad_neuro=0.3141e-4;% Radius of bouton; unit: cm; Ref: Calculated using vneuro
vneuro=(10^(-3))*(4/3)*pi*((rad_neuro)^3);% Volume of bouton; unit: L; Ref: Koester & Sakmann (2000)
sneuro=4*pi*(rad_neuro)^2;%Surface area of bouton; unit: cm^2; Ref: Koester & Sakmann (2000)
vr=-70;% Resting membrane potential of bouton; unit: mV
gna=120;% Sodium conductance density; unit: mS/cm^2; Ref: Hodgkin & Huxley (1952)
gk=36;% Pottasium conductance density; unit: mS/cm^2; Ref: Hodgkin & Huxley (1952)
p_ca=2.3e-9;% Conductance of single N-type channel; unit: mS; Ref: Weber et al (2010)
rho_ca=(4*8.0659e+007);% Density of N-type channels; unit: per cm^2; Ref: 
                     % Determined through computer simulations so that the 
                     % average Pr lies between 0.20.3 (when astrocyte is not stimulated) 
                     % similar to the experiments of Perea & Araque (2007)
gc=p_ca*rho_ca;% Calcium channel conductance density; unit: mS / cm^2
gl=0.3;% Leak conductance density; unit: mS/cm^2; Ref: Hodgkin & Huxley (1952)
vl=-59.4;% Reversal potential for leak; unit: mV; Ref: Hodgkin & Huxley (1952)
vna=45;% Reversal potential for Sodium ion; unit: mV; Ref: Hodgkin & Huxley (1952)
vk=-82;% Reversal potential for Pottasium ion; unit: mV; Ref: Hodgkin & Huxley (1952)
vca=125;% Reversal potential for Calcium ion determined through Nernst equation 
           % assuming an extracellular calcium concentration of 2 mM which
           % is equal to the extracellular calcium concentration in the
           % experiments of Perea & Araque (2007)
c1=0.185;% Ratio of Volume ER to Volume Neuron cell; unit: dimensionless; Ref: Shuai & Jung (2002)
v1=30*10^(-3);% Maximal calcium flux through IP3R; unit: per ms; Ref: Determined through Simulations
v2=0.2374*10^(-3);% Leak of calcium from ER to cytosol; unit: per ms; Ref: Determined through Simulations
v3=90;% Maximal SERCA pump rate; unit: nM/ms; Ref: Determined through Simulations
k3=0.1e+3;% Michaelis-Menten (MM) constant for SERCA; unit: nM; Ref: Erler et al (2004)
Ip=0.4;% Maximum PMCa current; unit: uA per cm2; Ref: Determined through computer simulations
k_pump=100;% MM constant for PMCa; unit: nM; Ref: Erler et al (2004)
v_leak=0.001022664392140;% Maximum leak of Ca2+ through plasma membrane; 
% unit: per ms; Ref: Determined through simulations
%IP3R-kinetics 
d1=0.13*10^(3);% IP3 dissociation constant; unit: nM; Ref: Shuai & Jung (2002)
d2=1.049*10^(3);% Inhibitory Ca2+ dissociation constant; unit: nM; Ref: Shuai & Jung (2002)
d3=0.9434*10^(3);% IP3 dissociation constant; unit: nM; Ref: Shuai & Jung (2002)
d5=0.08234*10^(3);% Activation Ca2+ dissociation constant; unit: nM; Ref: Shuai & Jung (2002)
a2=0.2*10^(-6);% Inhibitory Ca2+ binding constant; unit: per nM ms; Ref: Shuai & Jung (2002)
% Calcium current parameters
v_half=-17;% Half-activation voltage; unit: mV; Ref: Ishikawa et al (2005)
kca=8.4;% Slope factor; unit: mV; Ref: Ishikawa et al (2005)
taumc=10;% Time-constant; unit: ms; Ref: Ishikawa et al (2005)
% Synaptic vesicle parameter 
nv=2;% Number of docked vesicles; unit: dimensionless; Ref: Nikonenko & Skibo (2006)
gv=60;% Glutamate concentration in single vesicle; unit: mM; Ref: Montana et al. (2006)
kon=0.3e-3;% Calcium binding rate to vesicle; unit: per nM ms; Ref: Bollman et al (2000)
koff=3;% Calcium dissociation rate from vesicle; unit: per ms; Ref: Bollman et al (2000)
gamma=30;delta=8;% Isomerization constants; unit: per ms; Ref: Bollman et al (2000)
degG=10;% Extracellular glutamate degradation rate; unit: per ms; Ref: Destexhe et al (1998)
% Spontaneous release parameters (All determined through computer
% simulations). Spontaneous release of vesicles is determined by a poisson
% random variable with rate lambda.
sa1=(50e+3);% Calcium concentration at which lambda is halved; unit: nM
sa2=5e+3;% Constant which determines the slope of the sigmoidal shape of lambda; unit: nM
sa3=0.85;% Spontaneous vesicle release rate; unit: per ms
%IP3 generation parameters
v_glu=0.062;% Maximal IP3 production rate from mGluRs; unit: nM per ms; Ref: Nadkarni & Jung (2008)
k_glu=0.78e-3;% Glutamate concentration at which v_glu is halved; unit: mM; Ref: Nadkarni & Jung (2008)
tau_ip3=0.14e-3;% IP3 degradation constant; unit: per ms; Ref: Nadkarni & Jung (2008)
np=0.7;%0.3;
% Vesicle Regulatory time constants; unit: ms; Ref: Tsodyks & Markram (1998)
tau_rec=800;% Vesicle recovery time constant; unit: ms; Ref: Tsodyks & Markram (1997)
tau_inact=3;% Vesicle inactivation time constant; unit: ms; Ref: Tsodyks & Markram (1997)

%% Spine Parameters
R_in=0.7985e+8;% Input resistance of dendrite spine; unit: k-ohm; Ref: Calculated
tau_mem=50;% Post-synaptic membrane time constant; unit: ms; Ref: Tsodyks & Markram (1997)
rad_spine=0.6e-4;% Radius of spine-head; unit: cm; Ref: Dumitriu et al (2010)
vspine=(10^(-3))*(4/3)*pi*((rad_spine)^3);% Volume of spine-head assuming it to be spherical in shape; unit: L; Ref: Calculated
sspine=4*pi*(rad_spine)^2;% Surface area of spine-head assuming it to be spherical in shape; unit: cm^2; Ref: Calculated
v_post=-70;% Resting membrane potential of spine membrane; unit: mV
ks=100e-3;% Calcium extrusion rate by PMCa; unit: per ms; Ref: Keller et al (2008)
% AMPA Receptor Parameters all from Destexhe & Sejnowski (1998)
alpha_ampa=1.1;% Binding of transmitter to receptor; unit: mM per ms
beta_ampa=0.67;% Dissociation of transmitter from receptor; unit: per ms
g_ampa=0.35e-6;% Conductance of AMPAR; unit: mS

%% Astrocyte Parameters
% All values are from De Pitta et al (2009) except 'aNa.'
aNa=20;% Number of IP3 channels in cluster; unit: dimensionless; Ref: Nadkarni & Jung (2007)
ac0=2e+3;% Total cell free Ca2+ concentration; unit: nM
ac1=0.185;% Ratio of ER volume to cytosol volume; unit: dimensionless
av1=6e-3;% Maximal IP3R flux; unit: per ms
av2=0.11e-3;% Maximal rate of Ca2+ leak from ER; unit: per ms
av3=0.9;% Maximal rate of SERCA uptake; unit: nM per ms
ak3=0.1e+3;% SERCA Ca2+ affinity; unit: nM
ad1=0.13e+3;% IP3 dissociation constant; unit: nM
ad2=1.049e+3;% Ca2+ inactivation dissociation constant; unit: nM
ad3=0.9434e+3;% IP3 dissociation constant; unit: nM
ad5=0.08234e+3;% Ca2+ activation dissociation constant; unit: nM
aa2=0.2e-6;% IP3R binding rate for Ca2+ Inhibition; unit: per nM ms
% IP3 regulation parameters
av_plcd=0.05;% Maximal rate of IP3 production by PLC?; unit: nM per ms 
ak_plcd=1.5e+3;% Inhibition constant of PLC? activity; unit: nM
aK_plcd=0.1e+3;% Ca2+ affinity of PLC?; unit: nM
ar5p=0.05e-3;% Maximal rate of degradation by IP-5P; unit: per ms
av_plcb=0.5;% Maximal rate of IP3 production by PLC?; unit: nM per ms
aK_R=1.3e-3;% Glutamate affinity of the receptor; unit: mM
aK_P=10e-3;% Ca2+/PKC-dependent inhibition factor; unit: mM
aK_pi=0.6e+3;% Ca2+ affinity of PKC; unit: nM
av_3K=2;% Maximal rate of degradation by IP3-3K; unit: nM per ms
aK_D=0.7e+3;% Ca2+ affinity of IP3-3K; unit: nM
aK3=1e+3;% IP3 affinity of IP3-3K; unit: nM
% Gliotransmitter release parameters
nva=12;% Number of fusogenic SLMV inside astrocyte; unit: dimensionless; Ref: Malarkey & Parpura (2011)
gva=20;% Glutamate concentration inside one SLMV; unit: mM; Ref: Montana et al (2006)
ak1=3.75e-6;% Ca2+ association rate for S1; unit: per nM per ms; Ref: Bertram et al (1996)
ak2=2.5e-6;% Ca2+ association rate for S2; unit: per nM per ms; Ref: Bertram et al (1996)
akk3=1.25e-05;% Ca2+ association rate for S3; unit: per nM per ms; Ref: Computationally determined
ak_1=4e-4;% Ca2+ dissociation rate for S1; unit: per ms; Ref: Bertram et al (1996)
ak_2=1e-3;% Ca2+ dissociation rate for S2; unit: per ms; Ref: Bertram et al (1996)
ak_3=10e-3;% Ca2+ dissociation rate for S3; unit: per ms; Ref: Computationally determined
% Glutamate regulatory parameters
adegG=10;% Glutamate clearance rate from the extra-synaptic cleft; unit: per ms; Ref: Destexhe et al (1998)
atau_rec=800;% SLMV recovery time constant; unit: ms; Ref: Tsodyks & Markram (1997)
atau_inact=3;% SLMV inactivation time constant; unit: ms; Ref: Tsodyks & Markram (1997)

%% Other Misc. Parameters
tdr=0;% Records previous time at which synaptic vesicle release occured; unit: ms
R=8.314;% Real Gas constant; unit: J/K mole
T=273.15+21;% Temperature in the experiments of Perea & Araque (2007); unit: K
F=96487;% Faradays Constant; unit: Coulombs/mole

%% Variable Initialization
temp=zeros(1,tn+1);% Temporary array
Iapp=zeros(1,tn);% Applied current density
% Pre-synaptic Bouton Variables
x1=temp;% Temporal evolution of synaptic vesicle 1
x2=temp;% Temporal evolution of synaptic vesicle 2
G_syn=temp;% Synaptic glutamate concentration
c=temp;% Calcium concentration
cer=temp;% ER Calcium concentration
v=temp;% Membrane potential
m=temp;% Sodium channel activation 
h=temp;% Sodium channel inactivation
n=temp;% Pottassium channel activation
p=temp;% IP3 concentration
q=temp;% IP3 gating variable
mc=temp;% VGCC gating variable
R_syn=temp;% Fraction of releasable vesicles
E_syn=temp;% Fraction of effective vesicles in synaptic cleft
I_syn=temp;% Fraction of inactivated vesicles 
RRP=zeros(1,tn);% Vesicles ready to be released (spontaneous or evoked) 
% Astrocyte Variables
ca=temp;% Calcium concentration
ax=temp;% IP3R gating variable
ap=temp;% IP3 concentration
aO1=temp;% S1 with calcium bound 
aO2=temp;% S2 with calcium bound
aO3=temp;% S3 with calcium bound
aE_syn=temp;% Fraction of effective SLMV in extra-synaptic cleft 
aI_syn=temp;% Fraction of inactivated SLMV
aR_syn=temp;% Fraction of releasable SLMV
aG_syn=temp;% Glutamate concentration in extra-synaptic cleft
% Post-synaptic spine variables
V_post=temp;% Membrane potential
m_ampa=temp;% AMPAR gating variable
I_post=temp;% Current through AMPAR

%% Initial conditions
v(1)=-70;% Resting membrane potential of bouton; unit: mV
m(1)=0.1;% Gating variable for sodium channel (activation)
h(1)=0.6;% Gating variable for sodium channel (inactivation)
n(1)=0.3;% Gating variable for potassium channel (activation)
p(1)=160;% Resting [IP3]; unit: nM
c(1)=100;% Resting [Ca^{2+}]; unit: nM
cer(1)=(1.30e+6);% Resting ER [Ca^{2+}]; unit: nM
q(1)=0.22;% Gating variable for IP3R
mc(1)=0;% Gating variable for Calcium channel
mu=[1 0 0 0 0 0];% Inital vector for vesicle '1' & '2.' 
x1(1)=marko(mu);x2(1)=marko(mu);% Initial state of synaptic vesicle
R_syn(1)=1;% Relasable fraction of vesicles
E_syn(1)=0;% Effective fraction of vesicles in synaptic cleft
I_syn(1)=0;% Inactivated fraction of vesicles
G_syn(1)=1e-3;% Glutamate concentration in cleft; unit: mM
% Peri-synaptic Astrocyte variables
ca(1)=100;% Resting calcium concentration; unit: nM
ax(1)=0.5;% Gating variable for IP3R 
aO1(1)=0.48;% 1st calcium binding site of SLMV
aO2(1)=0.2;% 2nd calcium binding site of SLMV
aO3(1)=5e-4;% 3rd calcium binding site of SLMV
aE_syn(1)=0;% Fraction of effective SLMV in extra-synaptic cleft
aI_syn(1)=0;% Fraction of inactivated SLMV
aR_syn(1)=1;% Fraction of releasable SLMV
ap(1)=160;% Resting [IP3]; unit: nM
aG_syn(1)=1e-3;% Basal glutamate in the extra-synaptic cleft; unit: mM
% Post-synaptic spine variables
V_post(1)=-70;% Resting post-membrane potential; unit: mV
m_ampa(1)=0;% Gating variable of AMPAR
I_post(1)=0;% Current through AMPAR; unit: uA

for i=1:tn
    %% Applied Current Frequency
% Regular Spike: Here 'mod' is the standard modulus function. 't' is 
% in 'ms.' At present the spike frequency is 5 Hz with duration of 4 ms.    
    if mod(t(i),200)>=0 && mod(t(i),200)<=4
      Iapp(i)=10;% Applied current density
    else
      Iapp(i)=0;
    end;
    
    %% Bouton Processes
    lambda=sa3/(1+exp((sa1-c(i))/sa2));% Spontaneous vesicle release rate
    % Transition probability matrix 'P' governing calcium sensor state
    a21=5*kon*c(i)*deltaT;a12=koff*deltaT;a32=4*kon*c(i)*deltaT;
    a23=2*koff*deltaT;a43=3*kon*c(i)*deltaT;a34=3*koff*deltaT;
    a54=2*kon*c(i)*deltaT;a45=4*koff*deltaT;a65=kon*c(i)*deltaT;
    a56=5*koff*deltaT;a76=gamma*deltaT;a67=delta*deltaT;
    % Diagonal elements
    D0=1-a21;D1=1-a12-a32;D2=1-a23-a43;D3=1-a34-a54;D4=1-a45-a65;
    D5=1-a56-a76;D6=1-a67;
    % Matrix
    P=[D0,a12,0,0,0,0,0;
       a21,D1,a23,0,0,0,0;
        0,a32,D2,a34,0,0,0;
        0,0,a43,D3,a45,0,0;
        0,0,0,a54,D4,a56,0;
        0,0,0,0,a65,D5,a67;
        0,0,0,0,0,a76,D6];
    
    x1(i+1)=marko(P(:,x1(i)));% State vector for 1st vesicle
    x2(i+1)=marko(P(:,x2(i)));% State vector for 2nd vesicle
    
    % Condition to ensure that between two vesicular release there is a
    % refractory period of 6.34 ms. 'poissrnd' is a MATLAB function used 
    % to generate poisson random variable.  
    if poissrnd(lambda)==2 && abs(tdr-t(i))>6.34 && v(i)<=vr
        RRP(i)=1;
    elseif poissrnd(lambda)==1 && abs(tdr-t(i))>6.34 && v(i)<=vr
        RRP(i)=0.5;
        tdr=t(i);
    elseif (x1(i)==7 && x2(i)~=7 && abs(tdr-t(i))>6.34) && v(i)>vr
        RRP(i)=0.5;%when one vesicle is released
        tdr=t(i);
    elseif (x2(i)==7 && x1(i)~=7 && abs(tdr-t(i))>6.34) && v(i)>vr
        RRP(i)=0.5;%when one vesicle is released
        tdr=t(i);
    elseif (x1(i)==7 && x2(i)==7 && abs(tdr-t(i))>6.34) && v(i)>vr
        RRP(i)=1;%when both the vesicles are released
        tdr=t(i);
    else
        RRP(i)=0;
    end;
    
    % Gating Variables
    an=0.01*((-v(i)-60)/(exp((-v(i)-60)/10)-1));bn=0.125*exp((-v(i)-70)/80);% Opening closing parameters for activation of potassium channel
    am=0.1*((-v(i)-45)/(exp((-v(i)-45)/10)-1));bm=4*exp((-v(i)-70)/18);% Opening closing parameters for activation of sodium channel opening
    ah=0.07*exp((-v(i)-70)/20);bh=1/(exp((-v(i)-40)/10)+1);% Opening closing parameters for inactivation of sodium channel opening
    aq=a2*d2*(p(i)+d1)/(p(i)+d3);bq=a2*c(i);% Opening and closing parameters for IP3R gating
    mcinf=1/(1+exp((v_half-v(i))/kca));% VGCC current activation function
    % Ionic Currents
    ina=gna*(v(i)-vna);% Sodium current; unit: uA per cm^2
    ik=gk*(v(i)-vk);% Potassium current; unit: uA per cm^2
    il=gl*(v(i)-vl);% Leak current; unit: uA per cm^2
    ica=gc*(mc(i)^2)*(v(i)-vca);% VGCC current; unit: uA per cm^2
    I_pump=Ip*(c(i)^2/(c(i)^2+k_pump^2));% PMCa current; unit: uA per cm^2
    I_leak=v_leak*(v(i)-vca);% Calcium leak current; unit: uA per cm^2
    % IP3R Kinetics
    minf=p(i)/(p(i)+d1);ninf=c(i)/(c(i)+d5);% Steaty-state functions governing IP3 & Ca2+ based closing of IP3R
    jchan=c1*v1*(minf^3)*(ninf^3)*(q(i)^3)*(c(i)-cer(i));% Calcium flux through IP3R
    jpump=v3*(c(i)^2)/(k3^2+c(i)^2);% Serca pump flux
    jleak=c1*v2*(c(i)-cer(i));% Calcium leak through ER into cytosol
    p_glu=v_glu*((aG_syn(i)^np)/(k_glu^np + aG_syn(i)^np));% IP3 production due to extra-synaptic glutamate
    
    %% Astrocyte Processes
    % Time-constants for three binding sites of SLMV
    atau1=(ak1*ca(i)+ak_1);% Time-constant for closure of O1; unit: per ms
    atau2=(ak2*ca(i)+ak_2);% Time-constant for closure of O2; unit: per ms
    atau3=(akk3*ca(i)+ak_3);% Time-constant for closure of O3; unit: per ms
    
    %IP3 production & degradation terms. All are from De Pitta et al (2009)
    aplcb_ca=1+(aK_P/aK_R)*(ca(i)/(ca(i)+aK_pi));% Calcium-dependent inhibition of 'ap_glu'
    ap_glu=av_plcb*(G_syn(i)^0.7)/((G_syn(i)^0.7)+(aK_R*aplcb_ca)^0.7);% Agonist-dependent IP3 production
    ap_plcd=av_plcd*(1/(1+(ap(i)/ak_plcd)))*(ca(i)^2)/((ca(i)^2)+aK_plcd^2);% Agonist-independent IP3 production
    ap_mapk=av_3K*((ca(i)^4)/((ca(i)^4)+aK_D^4))*(ap(i)/(ap(i)+aK3));% IP3 degradation due to IP3-3K
    ap_deg=ar5p*(ap(i));% IP3 degradation due to IP-5P
    
    % IP3R gating variables
    aaq=aa2*ad2*(ap(i)+ad1)/(ap(i)+ad3);abq=aa2*ca(i);% gating variables for IP3R
    aminf=ap(i)/(ap(i)+ad1);aninf=ca(i)/(ca(i)+ad5);% Steaty-state functions governing IP3 & Ca2+ based closing of IP3R    
    auer=(ac0-ca(i))/ac1;% ER Calcium concentration

    % Using Box-Muller Algorithm (Fox, 1997) to simulate noise-term in 
    % equation (12) of the manuscript
    au1=rand(1,1);au2=rand(1,1);% Two uniformaly distributed random variables
    aa=((aaq*(1-ax(i))-abq*ax(i)))/aNa;% Calculating co-variance at each time-step 
    dW=sqrt(-(2*deltaT*(aa)*log(au1)))*cos(2*pi*au2);% Generating independent Gaussian random number
    % Because Gaussian random numbers can have arbitrarily large positive 
    % or negative values, it is possible that in the course of the 
    % simulation a ax value will fall outside the range of [0,1].Thus a
    % restriction has been imposed to chose those values of dW for which ax 
    % lies in the range [0,1] and discard all other values.
    if ax(i)>=0 && ax(i)<=1 
        ax(i+1)=deltaT*(aaq*(1-ax(i))-abq*ax(i))+ax(i)+dW;
    else
        ax(i+1)=deltaT*(aaq*(1-ax(i))-abq*ax(i))+ax(i);
    end;
    
    ajchan=ac1*av1*(aminf^3)*(aninf^3)*((abs(ax(i)))^3)*(ca(i)-auer);% Calcium flux through IP3R
    ajpump=av3*(ca(i)^2)/(ak3^2+ca(i)^2);% SERCA pump flux
    ajleak=ac1*av2*(ca(i)-auer);% Calcium leak from cytosol into ER
    
    %% Spine Processes
    I_post(i)=g_ampa*m_ampa(i)*(V_post(i));% AMPAR current; unit: uA
   
    %% Bouton DF
    p(i+1)=p(i)+deltaT*(((p(1)-p(i))*(tau_ip3))+p_glu);% Intracellular [IP3]
    m(i+1)=m(i)+deltaT*(am*(1-m(i))-bm*m(i));% Sodium channel activation
    h(i+1)=h(i)+deltaT*(ah*(1-h(i))-bh*h(i));% Sodium channel inactivation
    n(i+1)=n(i)+deltaT*(an*(1-n(i))-bn*n(i));% Potassium channel activation
    q(i+1)=q(i)+deltaT*(aq*(1-q(i))-bq*q(i));% IP3R gating variable
    mc(i+1)=mc(i)+deltaT*((mcinf-mc(i))/taumc);% VGCC gating variable
    
    v(i+1)=v(i)+deltaT*(Iapp(i)-((m(i)^3)*h(i)*ina+(n(i)^4)*ik+il));% Membrane potential displacemnt from rest 
    c(i+1)=c(i)+deltaT*(((-ica-I_pump-I_leak)*sneuro)/(2*F*vneuro)-jpump-jleak-jchan);% Intracellular calcium 
    cer(i+1)=cer(i)+deltaT*((jchan+jleak+jpump)/c1);% ER calcium concentration
    
    %Fraction of Neuronal Synaptic vesicles in releasable, effective
    %and inactive states respectively
    R_syn(i+1)=R_syn(i)+deltaT*(((I_syn(i))/tau_rec)-((RRP(i))*R_syn(i)));
    E_syn(i+1)=E_syn(i)+deltaT*(((RRP(i))*R_syn(i))-(E_syn(i)/tau_inact));
    I_syn(i+1)=1-R_syn(i+1)-E_syn(i+1);
    
    % Glutamate in the synaptic cleft
    G_syn(i+1)=G_syn(i)+deltaT*(nv*gv*E_syn(i)-degG*(G_syn(i)));

    %% Spine DF
    m_ampa(i+1)=m_ampa(i)+deltaT*(alpha_ampa*G_syn(i)*(1-m_ampa(i))-beta_ampa*m_ampa(i));% AMPAR gating variable
    V_post(i+1)=V_post(i)+deltaT*((-(V_post(i)-v_post)-R_in*(I_post(i)))/tau_mem);% Membrane potential
    
    %% Astrocyte DF
% This part of the code can be commented or uncommented depending upon 
% the need of simulation e.g. if uncommented with the present setting, it 
% will assume astrocyte calcium clamped till t=30 seconds and removes 
% the calcium clamp after t=30 seconds.    
%     if t(i)>=0 && t(i)<=30e+3
%         ca(i+1)=ca(1);% Calcium clamped to its resting concentration
%     elseif t(i)>30e+3 && t(i)<=600e+3
        ca(i+1)=ca(i)+(-ajchan-ajpump-ajleak)*deltaT;% Dynamic calcium concentration
%     end;
    ap(i+1)=ap(i)+deltaT*(ap_glu+ap_plcd-ap_mapk-ap_deg);% IP3 concentration
    aO1(i+1)=aO1(i)+deltaT*(ak1*ca(i)-(aO1(i)*atau1));% Site 1 of SLMV release machinery with calcium bound
    aO2(i+1)=aO2(i)+deltaT*(ak2*ca(i)-(aO2(i)*atau2));% Site 2 of SLMV release machinery with calcium bound
    aO3(i+1)=aO3(i)+deltaT*(akk3*ca(i)-(aO3(i)*atau3));% Site 3 of SLMV release machinery with calcium bound
    aRRP=aO1(i)*aO2(i)*aO3(i);% Releasable SLMVs in response to calcium increase
    aR_syn(i+1)=aR_syn(i)+deltaT*(((aI_syn(i))/atau_rec)-((heaviside(ca(i)-196.69)*aRRP)*aR_syn(i)));% Fraction of releasable SLMVs
    aE_syn(i+1)=aE_syn(i)+deltaT*(((heaviside(ca(i)-196.69)*aRRP)*aR_syn(i))-(aE_syn(i)/atau_inact));% Fraction of effective SLMVs
    aI_syn(i+1)=1-aR_syn(i+1)-aE_syn(i+1);% Fraction of inactivated SLMVs
    % Glutamate concentration in the extra-synaptic cleft
    aG_syn(i+1)=aG_syn(i)+deltaT*(nva*gva*aE_syn(i)-adegG*(aG_syn(i)));

end;
%% Display Results & Total time elapsed
subplot(2,3,1),plot(t,p,'k','Linewidth',2),title('Bouton IP_3','Fontsize',14)
subplot(2,3,2),plot(t,c,'k','Linewidth',2),title('Bouton Calcium','Fontsize',14)
subplot(2,3,3),plot(t,G_syn,'k','Linewidth',2),title('Synaptic Glutamate','Fontsize',14)
subplot(2,3,4),plot(t,ap,'k','Linewidth',2),title('Astrocyte IP_3','Fontsize',14)
subplot(2,3,5),plot(t,ca,'k','Linewidth',2),title('Astrocyte Calcium','Fontsize',14)
subplot(2,3,6),plot(t,aG_syn,'k','Linewidth',2),title('Extra-synaptic Glutamate','Fontsize',14)
toc;% To display total time elapsed